@@ -144,19 +144,28 @@ setpeer(argc, argv)
 
 		    destsp = getservbyname(argv[2], "tcp");
 
-		    if (destsp == NULL ) {
-			printf("%s: bad port name-- %s\n", argv[1], argv[2]);
-			printf ("usage: %s host-name [port]\n", argv[0]);
-			code = -1;
-			return;
-		    } else {
-			port = destsp->s_port;
+		    /* if lookup fails for ssl-ftp we fallback to
+		     * the default (unofficial) port number
+		     */
+		    if ((strcmp(argv[2],"ssl-ftp")==0) && (destsp==NULL))
+		        port = 150;
+		    else {
+			if (destsp == NULL ) {
+			    printf("%s: bad port name-- %s\n",argv[1],argv[2]);
+			    printf ("usage: %s host-name [port]\n",argv[0]);
+			    code = -1;
+			    return;
+			} else {
+			    port = ntohs(destsp->s_port);
+			}
 		    }
-
 		} else 
 #endif /* USE_SSL */
 		port = atoi(argv[2]);
-		if (port <= 0) {
+		/*
+		printf("%s: port %s=%d %d\n", argv[1], argv[2],port,htons(port));
+		*/
+		if (port == 0) {
 			printf("%s: bad port number-- %s\n", argv[1], argv[2]);
 			printf ("usage: %s host-name [port]\n", argv[0]);
 			code = -1;
