@@ -68,7 +68,11 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 #include <netdb.h>
 #include <fcntl.h>
 #include <pwd.h>
-#include <varargs.h>
+#ifdef USE_STDARG
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
 
 #if defined(SOLARIS2) || defined(__sgi) || defined(HPUX)
 #include <string.h>
@@ -319,11 +323,17 @@ cmdabort()
 }
 
 /*VARARGS*/
+int
+#ifdef USE_STDARG
+command(const char *fmt, ...)
+{
+#else
 command(va_alist)
-va_dcl
+	va_dcl
 {
-	va_list ap;
 	char *fmt;
+#endif
+	va_list ap;
 	int r;
 	void (*oldintr)();
 	void cmdabort();
@@ -334,7 +344,11 @@ va_dcl
 	abrtflag = 0;
 	if (debug) {
 		printf("---> ");
+#ifdef USE_STDARG
+		va_start(ap, fmt);
+#else
 		va_start(ap);
+#endif
 		fmt = va_arg(ap, char *);
 		if (strncmp("PASS ", fmt, 5) == 0)
 			printf("PASS XXXX");
@@ -350,7 +364,11 @@ va_dcl
 		return (0);
 	}
 	oldintr = signal(SIGINT, cmdabort);
+#ifdef USE_STDARG
+	va_start(ap, fmt);
+#else
 	va_start(ap);
+#endif
 	fmt = va_arg(ap, char *);
 
 #ifdef USE_SSL
@@ -398,6 +416,7 @@ getreply(expecteof)
 	int pflag = 0;
 	char *pt = pasv;
 	void cmdabort();
+	char buf[16];
 
 	oldintr = signal(SIGINT, cmdabort);
 	for (;;) {
@@ -409,23 +428,26 @@ getreply(expecteof)
 				case WILL:
 				case WONT:
 					c = GETC(cin);
+					sprintf(buf,
+					        "%c%c%c", IAC, DONT, c);
 #ifdef USE_SSL
-					FPRINTF(ssl_con,cout,
-					        "%c%c%c", IAC, DONT, c);
-#else /* !USE_SSL */
-					fprintf(cout,
-					        "%c%c%c", IAC, DONT, c);
+					if (ssl_active_flag)
+						SSL_write(ssl_con,buf,3);
+					else
 #endif /* !USE_SSL */
+						fwrite(buf,3,1,cout);
 					(void) FFLUSH(cout);
 					break;
 				case DO:
 				case DONT:
 					c = GETC(cin);
+					sprintf(buf, "%c%c%c", IAC, WONT, c);
 #ifdef USE_SSL
-					FPRINTF(ssl_con,cout, "%c%c%c", IAC, WONT, c);
-#else /* !USE_SSL */
-					fprintf(cout, "%c%c%c", IAC, WONT, c);
+					if (ssl_active_flag)
+						SSL_write(ssl_con,buf,3);
+					else
 #endif /* !USE_SSL */
+						fwrite(buf,3,1,cout);
 					(void) FFLUSH(cout);
 					break;
 				default:
@@ -1321,7 +1343,6 @@ dataconn(lmode)
 	    set_ssl_trace(ssl_data_con);
 
 	    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
-	    SSL_set_pref_cipher(ssl_data_con,getenv("SSL_CIPHER"));
 
 	    /* this is the "magic" call that makes 
 	     * this quick assuming Eric has this going
@@ -1353,10 +1374,9 @@ dataconn(lmode)
 		close(data);
 		return NULL;
 	    } else {
-		if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		    fprintf(SSL_LOG,"[SSL DATA Cipher %s]\n",
+		if (ssl_debug_flag) {
+		    BIO_printf(bio_err,"[SSL DATA Cipher %s]\n",
 			            SSL_get_cipher(ssl_con));
-		    fflush(SSL_LOG);
 		}
 		ssl_data_active_flag=1;
 	    }
@@ -1783,7 +1803,6 @@ char *u,*p;
 		set_ssl_trace(ssl_con);
 
 		SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-		SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
 
 		/* Add in any certificates if you want to here ... */
 		if (my_ssl_cert_file) {
@@ -1886,9 +1905,8 @@ ssl_getc(SSL *ssl_con)
       }
       return -1;
     } else {
-	if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-	    fprintf(SSL_LOG,"ssl_getc: SSL_read %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
-	    fflush(SSL_LOG);
+	if (ssl_debug_flag) {
+	    BIO_printf(bio_err,"ssl_getc: SSL_read %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
 	}
 	return onebyte & 0xff;
     }
@@ -1903,9 +1921,8 @@ int ssl_putc_flush(SSL *ssl_con)
 {
     if (putc_buf_pos>0) {
 	if (SSL_write(ssl_con,putc_buf,putc_buf_pos)!=putc_buf_pos) {
-	    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		fprintf(SSL_LOG,"ssl_putc_flush: WRITE FAILED\n");
-		fflush(SSL_LOG);
+	    if (ssl_debug_flag) {
+		BIO_printf(bio_err,"ssl_putc_flush: WRITE FAILED\n");
 	    }
 	    putc_buf_pos=0;
 	    return -1;
