@@ -126,6 +126,13 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 
 #ifdef USE_SSL
 
+#include "../lib/sslapp.c"
+
+BIO *bio_err;
+SSL *ssl_data_con;
+int ssl_auto_login=0;
+
+#if 0
 #include "rsa.h"        /* extra ... */
 /*
 #include "asn1.h"
@@ -139,17 +146,16 @@ static char sccsid[] = "@(#)ftpd.c	5.40 (Berkeley) 7/2/91";
 
 SSL *ssl_con;
 SSL_CTX *ssl_ctx;
-SSL *ssl_data_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_secure_flag=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 int ssl_certsok_flag=0;
-int ssl_auto_login=0;
+#endif
 
 static char *auth_ssl_name=NULL;
-
+FILE *cin, *cout;
 
 int ssl_data_active_flag=0;
 
@@ -176,6 +182,8 @@ int reply();
 int lreply();
 #endif
 
+static int standalone=0;
+
 /*
  * File containing login names
  * NOT to be used on this machine.
@@ -256,29 +264,6 @@ main(argc, argv, envp)
 	char *cp;
 	char *optarg;
 
-	/*
-	 * LOG_NDELAY sets up the logging connection immediately,
-	 * necessary for anonymous ftp's that chroot and can't do it later.
-	 */
-	openlog("ftpd", LOG_PID | LOG_NDELAY, LOG_DAEMON);
-	addrlen = sizeof (his_addr);
-	if (getpeername(0, (struct sockaddr *)&his_addr, &addrlen) < 0) {
-		syslog(LOG_ERR, "getpeername (%s): %m",argv[0]);
-		exit(1);
-	}
-	addrlen = sizeof (ctrl_addr);
-	if (getsockname(0, (struct sockaddr *)&ctrl_addr, &addrlen) < 0) {
-		syslog(LOG_ERR, "getsockname (%s): %m",argv[0]);
-		exit(1);
-	}
-#ifdef IP_TOS
-#ifdef IPTOS_LOWDELAY
-	tos = IPTOS_LOWDELAY;
-	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
-		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
-#endif
-#endif
-	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
 	debug = 0;
 #ifdef SETPROCTITLE
 	/*
@@ -337,6 +322,9 @@ main(argc, argv, envp)
 		        if (strcmp(optarg, "debug") == 0 ) {
 			    ssl_debug_flag=1;
 			}
+		        if (strcmp(optarg, "verbose") == 0 ) {
+			    ssl_verbose_flag=1;
+			}
 		        if (strcmp(optarg, "ssl") == 0 ) {
 			    ssl_only_flag=1;
 			}
@@ -356,6 +344,9 @@ main(argc, argv, envp)
 		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
 			    my_ssl_key_file=optarg+strlen("key=");
 			}
+		        if (strncmp(optarg, "standalone=", strlen("standalone=")) == 0 ) {
+			    standalone=atoi(optarg+strlen("standalone="));
+			}
 			/* we have swallowed an extra arg */
 			argc--;
 			argv++;
@@ -383,20 +374,48 @@ nextopt:
 	    FILE *fp;
             char *filename;
 
+	    /* keep the macros that are common between the client
+	     * and the server happy 
+	     */
+	    cin=stdin;
+	    cout=stderr;
+
+	    /* do things the "default" way */
+	    if (my_ssl_cert_file==NULL) {
+		sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
+			    "ftpd.pem");
+		ssl_cert_file=ssl_file_path;
+	    } else {
+	    	ssl_cert_file=my_ssl_cert_file;
+	    }
+
+	    if (!do_ssleay_init(1)) {
+		fprintf(stderr,"ftpd: SSLeay initialisation failed\n");
+		fflush(stderr);
+		sleep(1);
+		exit(1);
+	    }
+
+#if 0
             /* if we are not running in debug then any error
              * stuff from SSL debug *must* not go down
              * the socket (which 0,1,2 are all pointing to by
              * default)
              */
             if (ssl_debug_flag) {
-		(void)freopen("ftpd.err","w",stderr);
-		SSL_ERR=stderr;
-		SSL_LOG=NULL;
-                SSL_debug("ftpd.log");
-		if (SSL_LOG==NULL) {
-		    SSL_LOG=fopen("ftpd.log","w");
-		    if (SSL_LOG==NULL)
-			SSL_LOG=stderr;
+		if (standalone) {
+		    SSL_ERR=stderr;
+		    SSL_LOG=stderr;
+		} else {
+		    (void)freopen("ftpd.err","w",stderr);
+		    SSL_ERR=stderr;
+		    SSL_LOG=NULL;
+		    SSL_debug("ftpd.log");
+		    if (SSL_LOG==NULL) {
+			SSL_LOG=fopen("ftpd.log","w");
+			if (SSL_LOG==NULL)
+			    SSL_LOG=stderr;
+		    }
 		}
             } else {
                 /* disable all the debug and trace */
@@ -422,7 +441,10 @@ nextopt:
 	     * errors but I haven't bothered at this stage
 	     * as that can wait
 	     */
-            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
+/*
+            if (!X509_set_default_verify_paths(ssl_ctx->cert)) 
+*/
+            if (!SSL_set_default_verify_paths(ssl_ctx)) {
 		fprintf(stderr,"ftpd: cannot set default path via X509_set_default_verify_paths\n");
 		fflush(stderr);
 		sleep(1);
@@ -488,11 +510,68 @@ nextopt:
 		fprintf(SSL_LOG,"ftpd: got private key\n");
 		fflush(SSL_LOG);
 	    }
+#endif
 
         }
 #endif /* USE_SSL */
 
-	(void) freopen("/dev/null", "w", stderr);
+	/* support debugging things sanely without having to
+         * fight with inetd --tjh
+	 */
+	if (standalone) {
+	    static struct sockaddr_in name, peer;
+	    static int peerlen;
+	    int client, sock;
+
+
+            sock = socket(AF_INET, SOCK_STREAM, 0);
+	    if (sock < 0) {
+		perror("ftpd: standalone socket");
+		exit(1);
+	    }
+	    name.sin_port=htons(standalone);
+	    name.sin_addr.s_addr=INADDR_ANY;
+	    if (bind(sock,&name,sizeof(name))==-1) {
+		perror("ftpd: standalone bind");
+		exit(1);
+	    }
+	    listen(sock,5);
+	    fprintf(stderr,"Waiting for connection on %d\n",standalone);
+	    fflush(stderr);
+	    client=accept(sock,&peer,&peerlen);
+	    fprintf(stderr,"Got connection on %d\n",standalone);
+	    fflush(stderr);
+	    /* move socket to stdin, stdout */
+	    dup2(client,0);
+	    dup2(client,1);
+        }
+
+	/*
+	 * LOG_NDELAY sets up the logging connection immediately,
+	 * necessary for anonymous ftp's that chroot and can't do it later.
+	 */
+	openlog("ftpd", LOG_PID | LOG_NDELAY, LOG_DAEMON);
+	addrlen = sizeof (his_addr);
+	if (getpeername(0, (struct sockaddr *)&his_addr, &addrlen) < 0) {
+		syslog(LOG_ERR, "getpeername (%s): %m",argv[0]);
+		exit(1);
+	}
+	addrlen = sizeof (ctrl_addr);
+	if (getsockname(0, (struct sockaddr *)&ctrl_addr, &addrlen) < 0) {
+		syslog(LOG_ERR, "getsockname (%s): %m",argv[0]);
+		exit(1);
+	}
+#ifdef IP_TOS
+#ifdef IPTOS_LOWDELAY
+	tos = IPTOS_LOWDELAY;
+	if (setsockopt(0, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
+		syslog(LOG_WARNING, "setsockopt (IP_TOS): %m");
+#endif
+#endif
+	data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
+
+	if (!standalone) 
+		(void) freopen("/dev/null", "w", stderr);
 	(void) signal(SIGPIPE, lostconn);
 	(void) signal(SIGCHLD, SIG_IGN);
 	if ((int)signal(SIGURG, myoob) < 0)
@@ -1035,8 +1114,72 @@ dataconn(name, size, mode)
 		    sizeof(int));
 #endif
 #endif
+#ifdef USE_SSL
+		/* time to negotiate SSL on the data connection ...
+		 * do this via SSL_accept (as we are still the server
+		 * even though things are started around the other way)
+		 * 
+		 * note: we really *must* make sure the session stuff
+		 *       is copied correctly as we cannot afford a full
+		 *       SSL negotiation for each data socket!
+		 */
+		/* TODO XXXX fill in the blanks :-)
+		 */
+		ssl_data_active_flag=0;
+		if (ssl_active_flag && ssl_encrypt_data) {
+		    /* do SSL */
+
+		    reply(150, "Opening %s mode SSL data connection for %s%s.",
+			 type == TYPE_A ? "ASCII" : "BINARY", name, sizebuf);
+
+		    if (ssl_data_con!=NULL) {
+		      SSL_free(ssl_data_con);
+		      ssl_data_con=NULL;
+		    }
+		    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
+
+		    /* copy session details ... */
+		    SSL_copy_session_id(ssl_data_con,ssl_con);
+
+		    /* for 0.5.2 - want to change the timeout value etc ... */
+
+		    SSL_set_fd(ssl_data_con,pdata);
+		    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
+
+		    /* if is "safe" to read ahead */
+		    /* SSL_set_read_ahead(ssl_data_con,1); /**/
+
+		    if (ssl_debug_flag)
+			BIO_printf(bio_err,"===>START SSL_accept on DATA\n");
+
+		    if (SSL_accept(ssl_data_con)<=0) {
+			static char errbuf[1024];
+
+			sprintf(errbuf,"ftpd: SSL_accept DATA error %s\n",
+				    ERR_error_string(ERR_get_error(),NULL));
+			perror_reply(425, errbuf);
+			/* abort time methinks ... */
+			fclose(file);
+			return NULL;
+		    } else {
+			if (ssl_debug_flag) {
+			    BIO_printf(bio_err,"[SSL DATA Cipher %s]\n",
+					    SSL_get_cipher(ssl_con));
+			}
+			ssl_data_active_flag=1;
+		    }
+
+		    if (ssl_debug_flag)
+			BIO_printf(bio_err,"===>DONE SSL_accept on DATA\n");
+
+		} else {
+		    reply(150, "Opening %s mode data connection for %s%s.",
+			 type == TYPE_A ? "ASCII" : "BINARY", name, sizebuf);
+		}
+#else /* !USE_SSL */
 		reply(150, "Opening %s mode data connection for %s%s.",
 		     type == TYPE_A ? "ASCII" : "BINARY", name, sizebuf);
+#endif /* USE_SSL */
 		return(fdopen(pdata, mode));
 	}
 	if (data >= 0) {
@@ -1103,10 +1246,8 @@ dataconn(name, size, mode)
 	    /* if is "safe" to read ahead */
             /* SSL_set_read_ahead(ssl_data_con,1); /**/
 
-	    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		fprintf(SSL_LOG,"===>START SSL_accept on DATA\n");
-		fflush(SSL_LOG);
-	    }
+	    if (ssl_debug_flag)
+		BIO_printf(bio_err,"===>START SSL_accept on DATA\n");
 
 	    if (SSL_accept(ssl_data_con)<=0) {
                 static char errbuf[1024];
@@ -1118,18 +1259,14 @@ dataconn(name, size, mode)
 		fclose(file);
 		return NULL;
 	    } else {
-		if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		    fprintf(SSL_LOG,"[SSL DATA Cipher %s]\n",
+		if (ssl_debug_flag)
+		    BIO_printf(bio_err,"[SSL DATA Cipher %s]\n",
 			            SSL_get_cipher(ssl_con));
-		    fflush(SSL_LOG);
-		}
 		ssl_data_active_flag=1;
 	    }
 
-	    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		fprintf(SSL_LOG,"===>DONE SSL_accept on DATA\n");
-		fflush(SSL_LOG);
-	    }
+	    if (ssl_debug_flag) 
+		BIO_printf(bio_err,"===>DONE SSL_accept on DATA\n");
 
 	} else {
 	    reply(150, "Opening %s mode data connection for %s%s.",
@@ -1440,10 +1577,8 @@ va_dcl
 	vsprintf(outputbuf+strlen(outputbuf),fmt,ap);
 	strcat(outputbuf,"\r\n");
 
-	if (ssl_debug_flag && (SSL_LOG!=NULL) ) {
-	    fprintf(SSL_LOG,"\n<--- %s",outputbuf);
-	    fflush(SSL_LOG);
-	}
+	if (ssl_debug_flag)
+	    BIO_printf(bio_err,"\n<--- %s",outputbuf);
 
 	if (ssl_active_flag) {
 	    SSL_write(ssl_con,outputbuf,strlen(outputbuf));
@@ -1502,10 +1637,8 @@ va_dcl
 	vsprintf(outputbuf+strlen(outputbuf),fmt,ap);
 	strcat(outputbuf,"\r\n");
 
-	if (ssl_debug_flag && (SSL_LOG!=NULL) ) {
-	    fprintf(SSL_LOG,"\n<--- %s",outputbuf);
-	    fflush(SSL_LOG);
-	}
+	if (ssl_debug_flag) 
+	    BIO_printf(bio_err,"\n<--- %s",outputbuf);
 
 	if (ssl_active_flag) {
 	    SSL_write(ssl_con,outputbuf,strlen(outputbuf));
@@ -1718,6 +1851,15 @@ passive()
 	int len;
 	register char *p, *a;
 
+	/* don't let passive be set unless we are logged in! 
+	 * - this stops one of the core-dump holes in ftpd
+	 * --tjh
+	 */
+        if (pw==NULL) {
+		perror_reply(425, "Open passive connection denied - not logged in");
+		return;
+	}
+
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 	if (pdata < 0) {
 		perror_reply(425, "Can't open passive connection");
@@ -1811,6 +1953,7 @@ send_file_list(whichfiles)
 	register char **dirlist, *dirname;
 	int simple = 0;
 	char *strpbrk();
+	char buf[1024];
 
 	if (strpbrk(whichfiles, "~{[*?") != NULL) {
 		extern char **ftpglob(), *globerr;
@@ -1863,8 +2006,14 @@ send_file_list(whichfiles)
 					return;
 				transflag++;
 			}
-			FPRINTF(ssl_data_con,dout, "%s%s\n", dirname,
+			sprintf(buf,"%s%s\n", dirname,
 				type == TYPE_A ? "\r" : "");
+#ifdef USE_SSL
+			if (ssl_active_flag)
+				SSL_write(ssl_data_con,buf,strlen(buf));
+		    	else
+#endif /* USE_SSL */
+				fwrite(buf,strlen(buf),1,dout);
 			byte_count += strlen(dirname) + 1;
 			continue;
 		} else if ((st.st_mode&S_IFMT) != S_IFDIR)
@@ -1897,12 +2046,19 @@ send_file_list(whichfiles)
 						return;
 					transflag++;
 				}
-				if (nbuf[0] == '.' && nbuf[1] == '/')
-					FPRINTF(ssl_data_con,dout, "%s%s\n", &nbuf[2],
+				if (nbuf[0] == '.' && nbuf[1] == '/') {
+				  sprintf(buf,"%s%s\n",&nbuf[2],
 						type == TYPE_A ? "\r" : "");
+				} else {
+				  sprintf(buf,"%s%s\n", nbuf,
+						type == TYPE_A ? "\r" : "");
+			    	}
+#ifdef USE_SSL
+				if (ssl_active_flag)
+				  SSL_write(ssl_data_con,buf,strlen(buf));
 				else
-					FPRINTF(ssl_data_con,dout, "%s%s\n", nbuf,
-						type == TYPE_A ? "\r" : "");
+#endif /* USE_SSL */
+				  fwrite(buf,strlen(buf),1,dout);
 				byte_count += strlen(nbuf) + 1;
 			}
 		}
@@ -2017,10 +2173,8 @@ do_ssl_start()
 {
     static char errstr[1024];
 
-    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-	fprintf(SSL_LOG,"do_ssl_start triggered\n");
-	fflush(SSL_LOG);
-    }
+    if (ssl_debug_flag)
+	BIO_printf(bio_err,"do_ssl_start triggered\n");
 
     /* do the SSL stuff now ... before we play with pty's */
     ssl_con=(SSL *)SSL_new(ssl_ctx);
@@ -2028,6 +2182,7 @@ do_ssl_start()
     /* we are working with stdin (inetd based) by default */
     SSL_set_fd(ssl_con,0);
 
+#if 0
     if (SSL_use_RSAPrivateKey(ssl_con,ssl_private_key)==0) {
         sprintf(errstr,"ftpd: SSL_use_RSAPrivateKey %s",ERR_error_string(ERR_get_error(),NULL));
 	perror_reply(421, errstr);
@@ -2039,6 +2194,7 @@ do_ssl_start()
 	perror_reply(421, errstr);
 	dologout(1);
     }
+#endif
 
     SSL_set_verify(ssl_con,ssl_verify_flag,
 	    ssl_certsok_flag ? verify_callback : NULL);
@@ -2094,10 +2250,8 @@ ssl_getc(SSL *ssl_con)
     if (SSL_read(ssl_con,&onebyte,1)!=1)
       return -1;
     else {
-	if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-	    fprintf(SSL_LOG,"ssl_getc: SSL_read %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
-	    fflush(SSL_LOG);
-	}
+	if (ssl_debug_flag)
+	    BIO_printf(bio_err,"ssl_getc: SSL_read %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
 	return onebyte & 0xff;
     }
 }
@@ -2111,10 +2265,8 @@ int ssl_putc_flush(SSL *ssl_con)
 {
     if (putc_buf_pos>0) {
 	if (SSL_write(ssl_con,putc_buf,putc_buf_pos)!=putc_buf_pos) {
-	    if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-		fprintf(SSL_LOG,"ssl_putc_flush: WRITE FAILED\n");
-		fflush(SSL_LOG);
-	    }
+	    if (ssl_debug_flag) 
+		BIO_printf(bio_err,"ssl_putc_flush: WRITE FAILED\n");
 	    putc_buf_pos=0;
 	    return -1;
 	}
@@ -2139,31 +2291,32 @@ ssl_putc(SSL *ssl_con,int oneint)
     return onebyte;
 }
 
-/* we really shouldn't have code like this! --tjh */
-int 
-old_ssl_putc(SSL *ssl_con,int oneint)
-{
-    char onebyte;
-
-    onebyte = oneint & 0xff;
-
-    if (SSL_write(ssl_con,&onebyte,1)!=1)
-      return -1;
-    else {
-	if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-	    fprintf(SSL_LOG,"ssl_putc: SSL_write %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
-	    fflush(SSL_LOG);
-	}
-	return onebyte & 0xff;
-    }
-}
-
-static int
+static 
+int
+#ifdef SSLEAY8
+verify_callback(ok, ctx)
+int ok;
+X509_STORE_CTX *ctx;
+#else /* !SSLEAY8 */
 verify_callback(ok, xs, xi, depth, error)
 int ok;
 char *xs, *xi;
 int depth, error;
+#endif /* SSLEAY8 */
 {
+    static char *saved_subject=NULL;
+    X509 *peer;
+    char *subject, *issuer;
+#ifdef SSLEAY8
+    int depth,error;
+    char *xs;
+
+    depth=ctx->error_depth;
+    error=ctx->error;
+    xs=(char *)X509_STORE_CTX_get_current_cert(ctx);
+
+#endif /* SSLEAY8 */
+
     /*
      * If the verification fails, then don't remember the name.  However,
      * if we don't require a certificate, then return success which will
@@ -2176,11 +2329,17 @@ int depth, error;
 		free(auth_ssl_name);
  		auth_ssl_name = 0;
  	}
- 	return ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? 0 : 1;
+ 	ok=ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT ? 0 : 1;
+	goto done;
      }
      if (depth == 0)
  	auth_ssl_name =
- 	    (char *)X509_NAME_oneline(X509_get_subject_name(xs));
+ 	    (char *)ONELINE_NAME(X509_get_subject_name(xs));
+done: ;
+
+     if (ssl_debug_flag)
+     	BIO_printf(bio_err,"verify_callback: returning %d\n",ok);
+
      return ok;
 }
 
